@@ -55,6 +55,31 @@ YaraExpressionBuilder logicalFormula(std::vector<YaraExpressionBuilder> terms, c
 	return formula;
 }
 
+template <typename Op>
+YaraExpressionBuilder logicalFormula(std::vector<YaraExpressionBuilder> terms, std::vector<std::string> comments, const Op& op)
+{
+	if (terms.empty())
+		return boolVal(true);
+
+	if (terms.size() == 1)
+		return terms.front();
+
+	auto formula = op(terms[0], comments[0], terms[1]);
+	for (std::size_t i = 2; i < terms.size(); ++i)
+	{
+		if (!terms[i].canBeBool())
+		{
+			const auto& expr = terms[i].get();
+			error_handle("Expected boolean, got '" + expr->getText() + "' of type " + expr->getTypeString());
+		}
+		if (i >= 2)
+			formula = op(formula, comments[i-1], terms[i]);
+	}
+
+	formula.setType(Expression::Type::Bool);
+	return formula;
+}
+
 } //namespace
 
 /**
@@ -158,7 +183,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator<(const YaraExpressionBuil
 }
 
 /**
- * Applies less than or equal on two expression.
+ * Applies less than or equal on two expressions.
  *
  * @param other The other expression.
  *
@@ -175,7 +200,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator<=(const YaraExpressionBui
 }
 
 /**
- * Applies greater than on two expression.
+ * Applies greater than on two expressions.
  *
  * @param other The other expression.
  *
@@ -192,7 +217,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator>(const YaraExpressionBuil
 }
 
 /**
- * Applies greater than or equal on two expression.
+ * Applies greater than or equal on two expressions.
  *
  * @param other The other expression.
  *
@@ -209,7 +234,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator>=(const YaraExpressionBui
 }
 
 /**
- * Applies equals to on two expression.
+ * Applies equals to on two expressions.
  *
  * @param other The other expression.
  *
@@ -226,7 +251,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator==(const YaraExpressionBui
 }
 
 /**
- * Applies not equals to on two expression.
+ * Applies not equals to on two expressions.
  *
  * @param other The other expression.
  *
@@ -243,7 +268,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator!=(const YaraExpressionBui
 }
 
 /**
- * Applies plus on two expression.
+ * Applies plus on two expressions.
  *
  * @param other The other expression.
  *
@@ -262,7 +287,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator+(const YaraExpressionBuil
 }
 
 /**
- * Applies minus on two expression.
+ * Applies minus on two expressions.
  *
  * @param other The other expression.
  *
@@ -281,7 +306,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator-(const YaraExpressionBuil
 }
 
 /**
- * Applies multiplication on two expression.
+ * Applies multiplication on two expressions.
  *
  * @param other The other expression.
  *
@@ -300,7 +325,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator*(const YaraExpressionBuil
 }
 
 /**
- * Applies division on two expression.
+ * Applies division on two expressions.
  *
  * @param other The other expression.
  *
@@ -319,7 +344,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator/(const YaraExpressionBuil
 }
 
 /**
- * Applies modulo on two expression.
+ * Applies modulo on two expressions.
  *
  * @param other The other expression.
  *
@@ -329,7 +354,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator%(const YaraExpressionBuil
 {
 	bool will_be_float = _expr->isFloat() || other._expr->isFloat();
 
-	TokenIt token = _tokenStream->emplace_back(TokenType::MODULO, "%");
+	TokenIt token = _tokenStream->emplace_back(TokenType::PERCENT, "%");
 	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<ModuloExpression>(std::move(_expr), token, other.get());
@@ -338,7 +363,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator%(const YaraExpressionBuil
 }
 
 /**
- * Applies bitwise xor on two expression.
+ * Applies bitwise xor on two expressions.
  *
  * @param other The other expression.
  *
@@ -355,7 +380,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator^(const YaraExpressionBuil
 }
 
 /**
- * Applies bitwise and on two expression.
+ * Applies bitwise and on two expressions.
  *
  * @param other The other expression.
  *
@@ -373,7 +398,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator&(const YaraExpressionBuil
 }
 
 /**
- * Applies bitwise or on two expression.
+ * Applies bitwise or on two expressions.
  *
  * @param other The other expression.
  *
@@ -390,7 +415,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator|(const YaraExpressionBuil
 }
 
 /**
- * Applies bitwise shift left on two expression.
+ * Applies bitwise shift left on two expressions.
  *
  * @param other The other expression.
  *
@@ -407,7 +432,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator<<(const YaraExpressionBui
 }
 
 /**
- * Applies bitwise shift right on two expression.
+ * Applies bitwise shift right on two expressions.
  *
  * @param other The other expression.
  *
@@ -459,44 +484,29 @@ YaraExpressionBuilder& YaraExpressionBuilder::call(const std::vector<YaraExpress
  *
  * @return Builder.
  */
-YaraExpressionBuilder& YaraExpressionBuilder::comment(const std::string& message, bool multiline, const std::string& indent)
+YaraExpressionBuilder& YaraExpressionBuilder::comment(const std::string& message, bool multiline, const std::string& indent, bool linebreak)
 {
-	if (!message.empty())
-	{
-		TokenIt insert_before = _tokenStream->begin();
-		std::stringstream ss;
-		ss << indent;
-		if (multiline)
-		{
-			ss << "/*";
-			if (message.front() != '\n')
-				ss << " ";
-			for (auto c : message)
-			{
-				ss << c;
-				if (c == '\n')
-					ss << indent;
-			}
-			if (message.back() != '\n')
-				ss << " ";
-			ss << "*/";
-			_tokenStream->emplace(insert_before, TokenType::COMMENT, ss.str());
-		}
-		else
-		{
-			for (auto item : message)
-				if (item == '\n')
-					throw YaraExpressionBuilderError("Error: one-line comment must not contain \\n.");
-			ss << "// " << message;
-			_tokenStream->emplace(insert_before, TokenType::ONELINE_COMMENT, ss.str());
-		}
-		_tokenStream->emplace(insert_before, TokenType::NEW_LINE, "\n");
-	}
+	_tokenStream->comment(message, multiline, indent, linebreak);
 	return *this;
 }
 
 /**
- * Applies operation contains on two expression.
+ * Puts comment behind the expression.
+ *
+ * @param message The comment message.
+ * @param multiline If set, the commet will be multiline.
+ * @param indent Additional indent added to the indentation computed by the autoformatter.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder& YaraExpressionBuilder::commentBehind(const std::string& message, bool multiline, const std::string& indent, bool linebreak)
+{
+	_tokenStream->commentBehind(message, multiline, indent, linebreak);
+	return *this;
+}
+
+/**
+ * Applies operation contains on two expressions.
  *
  * @param other The other expression.
  *
@@ -513,7 +523,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::contains(const YaraExpressionBuild
 }
 
 /**
- * Applies operation matches on two expression.
+ * Applies operation matches on two expressions.
  *
  * @param other The other expression.
  *
@@ -530,7 +540,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::matches(const YaraExpressionBuilde
 }
 
 /**
- * Applies operation iequals on two expression.
+ * Applies operation iequals on two expressions.
  *
  * @param other The other expression.
  *
@@ -547,7 +557,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::iequals(const YaraExpressionBuilde
 }
 
 /**
- * Applies operation defined on two expression.
+ * Applies operation defined on the expression.
  *
  * @return Builder.
  */
@@ -558,6 +568,18 @@ YaraExpressionBuilder& YaraExpressionBuilder::defined() {
 	return *this;
 }
 
+/**
+ * Applies operation percent on the expression.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder& YaraExpressionBuilder::percent() {
+	auto token = _tokenStream->emplace_back(TokenType::PERCENT, "%");
+	_expr = std::make_shared<PercentualExpression>(std::move(_expr), token);
+	setType(Expression::Type::Int);
+	return *this;
+}
+
 /**
  * Accesses the attribute of a structure expression.
  *
@@ -791,10 +813,19 @@ YaraExpressionBuilder boolVal(bool value)
 YaraExpressionBuilder id(const std::string& id)
 {
 	auto ts = std::make_shared<TokenStream>();
-	const std::shared_ptr<Symbol>& symbol = std::make_shared<ValueSymbol>(id, Expression::Type::Object);
-	TokenIt token = ts->emplace_back(TokenType::ID, std::move(symbol));
-	auto expression = std::make_shared<IdExpression>(token);
-	return YaraExpressionBuilder(std::move(ts), std::move(expression));
+	if (endsWith(id, '*'))
+	{
+		TokenIt token = ts->emplace_back(TokenType::ID_WILDCARD, id.substr(0, id.length() - 1));
+		TokenIt wildcard = ts->emplace_back(TokenType::ID_WILDCARD, '*');
+		return YaraExpressionBuilder(std::move(ts), std::make_shared<IdWildcardExpression>(token, wildcard));
+	}
+	else
+	{
+		const std::shared_ptr<Symbol>& symbol = std::make_shared<ValueSymbol>(id, Expression::Type::Object);
+		TokenIt token = ts->emplace_back(TokenType::ID, std::move(symbol));
+		auto expression = std::make_shared<IdExpression>(token);
+		return YaraExpressionBuilder(std::move(ts), std::move(expression));
+	}
 }
 
 /**
@@ -1099,6 +1130,29 @@ YaraExpressionBuilder of(const YaraExpressionBuilder& ofExpr, const YaraExpressi
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
 }
 
+/**
+ * Creates the expression with match of the specified set of strings in given range.
+ *
+ * @param quantifier All / Any / None expression.
+ * @param set Set expression.
+ * @param range Range expression.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder of(const YaraExpressionBuilder& quantifier, const YaraExpressionBuilder& set, const YaraExpressionBuilder& range)
+{
+	auto ts = std::make_shared<TokenStream>();
+	ts->moveAppend(quantifier.getTokenStream());
+	auto ofToken = ts->emplace_back(TokenType::OF, "of");
+	ts->moveAppend(set.getTokenStream());
+	auto inToken = ts->emplace_back(TokenType::OP_IN, "in");
+	ts->moveAppend(range.getTokenStream());
+
+	auto expression = std::make_shared<OfExpression>(quantifier.get(), ofToken, set.get(), inToken, range.get());
+
+	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
+}
+
 /**
  * Creates an iterable array of elements.
  *
@@ -1135,14 +1189,14 @@ YaraExpressionBuilder iterable(const std::vector<YaraExpressionBuilder>& element
 YaraExpressionBuilder set(const std::vector<YaraExpressionBuilder>& elements)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt lb = ts->emplace_back(TokenType::LP, "(");
+	TokenIt lb = ts->emplace_back(TokenType::LP_ENUMERATION, "(");
 	for (std::size_t i = 0; i < elements.size(); ++i)
 	{
 		ts->moveAppend(elements[i].getTokenStream());
 		if (i < elements.size() - 1)
 			ts->emplace_back(TokenType::COMMA, ",");
 	}
-	TokenIt rb = ts->emplace_back(TokenType::RP, ")");
+	TokenIt rb = ts->emplace_back(TokenType::RP_ENUMERATION, ")");
 
 	std::vector<Expression::Ptr> elementsExprs;
 	std::for_each(elements.begin(), elements.end(), [&elementsExprs](const YaraExpressionBuilder& expr) { elementsExprs.push_back(expr.get()); });
@@ -1205,6 +1259,58 @@ YaraExpressionBuilder disjunction(const YaraExpressionBuilder& lhs, const YaraEx
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
 }
 
+/**
+ * Creates conjunction.
+ *
+ * @param lhs Left-hand side.
+ * @param lhscomment First comment.
+ * @param rhs Right-hand side.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder conjunction(const YaraExpressionBuilder& lhs, const std::string& lhscomment, const YaraExpressionBuilder& rhs)
+{
+	if (!lhs.canBeBool())
+		error_handle(ArgType::Left, "and", "bool", rhs.get());
+	else if (!rhs.canBeBool())
+		error_handle(ArgType::Right, "and", "bool", lhs.get());
+
+	auto ts = std::make_shared<TokenStream>();
+	ts->moveAppend(lhs.getTokenStream());
+	TokenIt andToken = ts->emplace_back(TokenType::AND, "and");
+	ts->commentBehind(lhscomment, false, "", true);
+	ts->moveAppend(rhs.getTokenStream());
+
+	auto expression = std::make_shared<AndExpression>(lhs.get(), andToken, rhs.get(), true);
+	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
+}
+
+/**
+ * Creates disjunction.
+ *
+ * @param lhs Left-hand side.
+ * @param lhscomment First comment.
+ * @param rhs Right-hand side.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder disjunction(const YaraExpressionBuilder& lhs, const std::string& lhscomment, const YaraExpressionBuilder& rhs)
+{
+	if (!lhs.canBeBool())
+		error_handle(ArgType::Left, "or", "bool", lhs.get());
+	else if (!rhs.canBeBool())
+		error_handle(ArgType::Right, "or", "bool", rhs.get());
+
+	auto ts = std::make_shared<TokenStream>();
+	ts->moveAppend(lhs.getTokenStream());
+	TokenIt orToken = ts->emplace_back(TokenType::OR, "or");
+	ts->commentBehind(lhscomment, false, "", true);
+	ts->moveAppend(rhs.getTokenStream());
+
+	auto expression = std::make_shared<OrExpression>(lhs.get(), orToken, rhs.get(), true);
+	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
+}
+
 /**
  * Creates conjunction of terms.
  *
@@ -1245,17 +1351,23 @@ YaraExpressionBuilder disjunction(const std::vector<YaraExpressionBuilder>& term
  *
  * @return Builder.
  */
-YaraExpressionBuilder conjunction(const std::vector<std::pair<YaraExpressionBuilder, std::string>>& terms)
+YaraExpressionBuilder conjunction(const std::vector<std::pair<YaraExpressionBuilder, std::string>>& commented_terms)
 {
-	std::vector<YaraExpressionBuilder> commented_terms;
-	for (const auto& pair : terms)
+	std::vector<YaraExpressionBuilder> terms;
+	std::vector<std::string> comments;
+	for (const auto& pair : commented_terms)
 	{
 		if (!pair.first.canBeBool())
 			error_handle(ArgType::Single, "and", "bool", pair.first.get());
-		commented_terms.push_back(pair.first);
-		commented_terms.back().comment(pair.second, true);
+		terms.push_back(pair.first);
+		comments.push_back(pair.second);
 	}
-	return logicalFormula(commented_terms, [](YaraExpressionBuilder& term1, YaraExpressionBuilder& term2) { return conjunction(term1, term2, true); });
+	auto output = logicalFormula(terms, comments, []( YaraExpressionBuilder& term1, std::string& comment1, YaraExpressionBuilder& term2) {
+		return conjunction(term1, comment1, term2);
+	});
+
+	output.commentBehind(comments.back(), false, "", false);
+	return output;
 }
 
 /**
@@ -1266,17 +1378,23 @@ YaraExpressionBuilder conjunction(const std::vector<std::pair<YaraExpressionBuil
  *
  * @return Builder.
  */
-YaraExpressionBuilder disjunction(const std::vector<std::pair<YaraExpressionBuilder, std::string>>& terms)
+YaraExpressionBuilder disjunction(const std::vector<std::pair<YaraExpressionBuilder, std::string>>& commented_terms)
 {
-	std::vector<YaraExpressionBuilder> commented_terms;
-	for (const auto& pair : terms)
+	std::vector<YaraExpressionBuilder> terms;
+	std::vector<std::string> comments;
+	for (const auto& pair : commented_terms)
 	{
 		if (!pair.first.canBeBool())
-			error_handle(ArgType::Single, "or", "bool", pair.first.get());
-		commented_terms.push_back(pair.first);
-		commented_terms.back().comment(pair.second, true);
+			error_handle(ArgType::Single, "and", "bool", pair.first.get());
+		terms.push_back(pair.first);
+		comments.push_back(pair.second);
 	}
-	return logicalFormula(commented_terms, [](YaraExpressionBuilder& term1, YaraExpressionBuilder& term2) { return disjunction(term1, term2, true); });
+	auto output = logicalFormula(terms, comments, []( YaraExpressionBuilder& term1, std::string& comment1, YaraExpressionBuilder& term2) {
+		return disjunction(term1, comment1, term2);
+	});
+
+	output.commentBehind(comments.back(), false, "", false);
+	return output;
 }
 
 /**
@@ -1348,6 +1466,18 @@ YaraExpressionBuilder any()
 	return YaraExpressionBuilder(std::move(ts), std::make_shared<AnyExpression>(token));
 }
 
+/**
+ * Creates the expression with keyword @c any.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder none()
+{
+	auto ts = std::make_shared<TokenStream>();
+	TokenIt token = ts->emplace_back(TokenType::NONE, "none");
+	return YaraExpressionBuilder(std::move(ts), std::make_shared<NoneExpression>(token));
+}
+
 /**
  * Creates the expression with keyword @c them.
  *
