@@ -14,6 +14,7 @@
 #include <yaramod/builder/yara_expression_builder.h>
 #include <yaramod/builder/yara_hex_string_builder.h>
 #include <yaramod/builder/yara_rule_builder.h>
+#include <yaramod/types/expression.h>
 #include <yaramod/types/plain_string.h>
 #include <yaramod/types/token_type.h>
 #include <yaramod/yaramod.h>
@@ -275,7 +276,8 @@ void addBasicClasses(py::module& module)
 			})
 		.def("remove_imports", [](YaraFile& self, const std::function<bool(const std::shared_ptr<Module>&)>& pred) {
 				self.removeImports(pred);
-			});
+			})
+		.def("expand_rule_prefix_from_origin", &YaraFile::expandRulePrefixFromOrigin);
 
 	py::class_<Location>(module, "Location")
 		.def_property_readonly("file_path", &Location::getFilePath)
@@ -358,6 +360,7 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("structure", &Module::getStructure);
 
 	py::class_<String, std::shared_ptr<String>>(module, "String")
+		.def("remove_modifiers", &String::removeModifiers)
 		.def_property_readonly("text", &String::getText)
 		.def_property_readonly("pure_text", [](String& self) {
 			return py::bytes(self.getPureText());
@@ -376,6 +379,7 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("is_base64", &String::isBase64)
 		.def_property_readonly("is_base64_wide", &String::isBase64Wide)
 		.def_property_readonly("location", &String::getLocation)
+		.def_property_readonly("modifiers", &String::getModifiers)
 		.def_property_readonly("modifiers_text", &String::getModifiersText)
 		.def_property_readonly("token_id", &String::getIdentifierToken)
 		.def_property_readonly("token_assign", &String::getAssignToken)
@@ -537,6 +541,7 @@ void addExpressionClasses(py::module& module)
 		.def("accept", &Expression::accept)
 		.def("get_text", &Expression::getText, py::arg("indent") = std::string{})
 		.def("exchange_tokens", py::overload_cast<Expression*>(&Expression::exchangeTokens))
+		.def_property_readonly("uid", &Expression::getUid)
 		.def_property_readonly("text",
 				// getText() has default parameter and Python can't deal with it
 				[](const Expression* self) {
@@ -677,6 +682,10 @@ void addExpressionClasses(py::module& module)
 		.def_property("symbol",
 				&IdExpression::getSymbol,
 				&IdExpression::setSymbol);
+	exprClass<IdWildcardExpression>(module, "IdWildcardExpression")
+		.def_property("id",
+				&IdWildcardExpression::getId,
+				py::overload_cast<const std::string&>(&IdWildcardExpression::setId));
 	exprClass<StructAccessExpression, IdExpression>(module, "StructAccessExpression")
 		.def_property("structure",
 				&StructAccessExpression::getStructure,
